Performance testing client
==========================

### Steps to deploy
1. Choose an appropriate namespace `<namespace>` to use as Kubernetes namespace
   and to identify a Docker repository (see below for more on this).
2. Choose a tag `<tag>` to identify this specific flavour of the client. It is
   used to name the produced Docker image as well as to name the release of the
   Helm chart.
3. Choose a profile `<profile>`. Currently, it can be either `fanout-send` or
   `fanout-recv`.
4. Determine the server URL, `<server url>`. If might look something like this,
   `realm://sync-perf-test-server-<tag>:9090` ([see also](../server/README.md)).
5. Determine the Stitch application identifier, `<app id>`. It is probablu on
   the form, `default-tgpsj`. You will probably need to login to the Stitch
   server in order to find it.
6. Build a Docker image with the test client by running `sh build.sh <namespace>
   <tag>`. This requires access to Docker registry (see below).
7. Deploy the test client into Arena Kubernetes cluster by running `sh deploy.sh
   <namespace> <tag> <profile> <server url> <app id>` (see below for how to get
   access).

For example, if the namespace is `xx` the tag is `foo`, the profile is
`fanout-send`, the server url is `realm://sync-perf-test-server-foo:9090`, and
the application identifier is `default-tgpsj`, here are the commands needed to
build and deploy the client:

    sh build.sh xx foo
    sh deploy.sh xx foo fanout-send realm://sync-perf-test-server-foo:9090 default-tgpsj

The specified namespace, `<namespace>`, must be one of the existing Kubernetes
namespaces in the Arena cluster (see below on how to get access). You may need
to create a new namespace there (talk to Adam Lebsack). The namespace is also
used to name a unique Docker repository on Amazon Web Services. The Docker
repository path is
`012067661104.dkr.ecr.eu-west-1.amazonaws.com/test/sync-perf-<namespace>-client`,
which you will need to create (talk to Adam Lebsack), if it does not already
exist.

Rebuilding the Docker image requires access to the Docker registry at
`012067661104.dkr.ecr.eu-west-1.amazonaws.com`. To gain access, first configure
the AWS client using `aws configure` (set region to `eu-west-1`), then run the
Docker login command that is generated by running `aws ecr get-login
--registry-ids 012067661104`.

To gain access to the Arena cluster from your command line, first go to the
[Kubernetes web interface](https://arena.k8s.realmlab.net), and login. Then
update your local environment (`~/.kube/config`) by running the script displayed
on [this page](https://arena.k8s.realmlab.net/kubeconfig).

The fan-out dashboard is currently available on
https://stats.realmlab.net/d/lJYvjQBiz/fan-out?orgId=1.

### Scripts

| Name      | Description
|-----------|----------------------------------------------------------------------------
| build.sh  | Rebuild and publish the Docker image containing the client (`docker build`)
| deploy.sh | Deploy the client into Arena cluster (`helm upgrade`)
| status.sh | See status on client deployment (`helm status`)
| logs.sh   | Show log output from deployed client (`kubectl logs`)
| purge.sh  | Delete client deployment (`helm delete`)
| list.sh   | List all current client deployments in namespace (`helm list`)

These scripts have been tested with [Helm
2.16.7](https://github.com/helm/helm/releases/tag/v2.16.7).
